from .intersect import Features,get_feature_start_on_segment,get_feature_stop_on_segment,invert_seg

# find the occurence of the segment that corresponds to the feature's copy
def get_seg_occ(seg,walk,feat,copy,segments_on_target_genome):
    seg_in_walk=segments_on_target_genome[seg].get_all_seg(walk)
    for seg_occurence in seg_in_walk:
        for feat_copy in seg_occurence[5:]:
            if (feat_copy[0]==feat) & (feat_copy[1]==copy):
                return seg_occurence # [chr,start,stop,strand,index,copies]
    print("no occ found ???",seg,walk,feat,copy)
    print(segments_on_target_genome[seg].get_all_filenames())
    print(Features[feat].segments_list_source)
    print(Features[feat].segments_list_target)
    exit()

# get the start position of the features on the linear target genome, using their coordinates on the graph and the coordinantes of the segments on the genome
def get_feature_start_on_target_genome(start_seg,feat_id,walk,copy_id,segments_on_target_genome):
    seg_start_pos=get_seg_occ(start_seg,walk,feat_id,copy_id,segments_on_target_genome)[1]
    feat_start_pos=get_feature_start_on_segment(start_seg,feat_id)    
    return seg_start_pos+feat_start_pos-1

# get the stop position of the features on the linear target genome, using their coordinates on the graph and the coordinantes of the segments on the genome
def get_feature_stop_on_target_genome(stop_seg,feat_id,walk,copy_id,segments_on_target_genome):
    seg_start_pos=get_seg_occ(stop_seg,walk,feat_id,copy_id,segments_on_target_genome)[1]
    feat_stop_pos=get_feature_stop_on_segment(stop_seg,feat_id)
    return seg_start_pos+feat_stop_pos-1

# get the start position of the features on the linear target genome for inverted features
def get_feature_start_on_target_genome_inv(start_seg,feat_id,walk,copy_id,segments_on_target_genome):
    seg_end_pos=get_seg_occ(start_seg,walk,feat_id,copy_id,segments_on_target_genome)[2]
    feat_start_pos=get_feature_start_on_segment(start_seg,feat_id)
    return seg_end_pos-feat_start_pos+1

# get the stop position of the features on the linear target genome for inverted features
def get_feature_stop_on_target_genome_inv(stop_seg,feat_id,walk,copy_id,segments_on_target_genome):
    seg_end_pos=get_seg_occ(stop_seg,walk,feat_id,copy_id,segments_on_target_genome)[2]
    feat_stop_pos=get_feature_stop_on_segment(stop_seg,feat_id)
    return seg_end_pos-feat_stop_pos+1


# invert the given strand
def invert_strand(strand):
    match strand:
        case "+":
            return "-"
        case "-":
            return "+"
        case ">":
            return "<"
        case "<":
            return ">"
        case default:
            return ""
        
# invert all the segments in a list (change the orientation)
def invert_segment_list(seg_list):
    list_inverted=list()
    for seg in seg_list:
        list_inverted.append(invert_seg(seg))
    return list(reversed(list_inverted))


# outputs the sequence of an oriented segment
def get_segment_sequence(seg_info,segment):
    if segment[0]==">":
        return seg_info[segment[1:]][1]
    else:
        return reverse_complement(seg_info[segment[1:]][1])

# outputs the reverse complement of a sequence
def reverse_complement(sequence):
    sequence_rc=""
    for char in sequence:
        sequence_rc+=complement(char)
    return sequence_rc[::-1]

# outputs the reverse complement of a nucleotide
def complement(nucl):
    match nucl:
        case "A":
            return "T"
        case "C":
            return "G"
        case "G":
            return "C"
        case "T":
            return "A"
    return nucl